@@ -167,6 +167,7 @@ static void                 fm_icon_view_set_directory_tighter_layout (FMIconVie
 								       NautilusFile         *file,
 								       gboolean              tighter_layout);
 static gboolean              fm_icon_view_supports_manual_layout      (FMIconView           *icon_view);
+static void                 fm_icon_view_reveal_selection             (FMDirectoryView      *view);
 static const SortCriterion *get_sort_criterion_by_sort_type           (NautilusFileSortType  sort_type);
 static void                 set_sort_criterion_by_sort_type           (FMIconView           *icon_view,
 								       NautilusFileSortType  sort_type);
@@ -469,6 +470,8 @@ should_show_file_on_screen (FMDirectoryView *view, NautilusFile *file)
 static void
 fm_icon_view_remove_file (FMDirectoryView *view, NautilusFile *file, NautilusDirectory *directory)
 {
+	FMIconView *icon_view;
+
 	/* This used to assert that 'directory == fm_directory_view_get_model (view)', but that
 	 * resulted in a lot of crash reports (bug #352592). I don't see how that trace happens.
 	 * It seems that somehow we get a files_changed event sent to the view from a directory
@@ -490,8 +493,14 @@ fm_icon_view_remove_file (FMDirectoryView *view, NautilusFile *file, NautilusDir
 		g_free (model_uri);
 	}
 	
-	if (nautilus_icon_container_remove (get_icon_container (FM_ICON_VIEW (view)),
+	icon_view = FM_ICON_VIEW (view);
+
+	if (nautilus_icon_container_remove (get_icon_container (icon_view),
 					    NAUTILUS_ICON_CONTAINER_ICON_DATA (file))) {
+		if (file == icon_view->details->audio_preview_file) {
+			preview_audio (icon_view, NULL, FALSE);
+		}
+
 		nautilus_file_unref (file);
 	}
 }
@@ -1381,6 +1390,7 @@ set_sort_criterion_by_sort_type (FMIconView *icon_view,
 
 	set_sort_criterion (icon_view, sort);
 	nautilus_icon_container_sort (get_icon_container (icon_view));
+	fm_icon_view_reveal_selection (FM_DIRECTORY_VIEW (icon_view));
 }
 
 
@@ -1395,6 +1405,7 @@ action_reversed_order_callback (GtkAction *action,
 	if (set_sort_reversed (icon_view,
 			       gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))) {
 		nautilus_icon_container_sort (get_icon_container (icon_view));
+		fm_icon_view_reveal_selection (FM_DIRECTORY_VIEW (icon_view));
 	}
 }
 
@@ -1988,7 +1999,7 @@ preview_audio (FMIconView *icon_view, NautilusFile *file, gboolean start_flag)
 			
 	if (start_flag) {
 		icon_view->details->audio_preview_file = file;
-		icon_view->details->audio_preview_timeout = g_timeout_add (1000, play_file, icon_view);
+		icon_view->details->audio_preview_timeout = g_timeout_add_seconds (1, play_file, icon_view);
 	}
 }
 
@@ -2176,6 +2187,43 @@ fm_icon_view_screen_changed (GtkWidget *widget,
 	}
 }
 
+static gboolean
+fm_icon_view_scroll_event (GtkWidget *widget,
+			   GdkEventScroll *scroll_event)
+{
+	FMIconView *icon_view;
+	GdkEvent *event_copy;
+	GdkEventScroll *scroll_event_copy;
+	gboolean ret;
+
+	icon_view = FM_ICON_VIEW (widget);
+
+	if (icon_view->details->compact &&
+	    (scroll_event->direction == GDK_SCROLL_UP ||
+	     scroll_event->direction == GDK_SCROLL_DOWN)) {
+		ret = fm_directory_view_handle_scroll_event (FM_DIRECTORY_VIEW (icon_view), scroll_event);
+		if (!ret) {
+			/* in column-wise layout, re-emit vertical mouse scroll events as horizontal ones,
+			 * if they don't bump zoom */
+			event_copy = gdk_event_copy ((GdkEvent *) scroll_event);
+
+			scroll_event_copy = (GdkEventScroll *) event_copy;
+			if (scroll_event_copy->direction == GDK_SCROLL_UP) {
+				scroll_event_copy->direction = GDK_SCROLL_LEFT;
+			} else {
+				scroll_event_copy->direction = GDK_SCROLL_RIGHT;
+			}
+
+			ret = gtk_widget_event (widget, event_copy);
+			gdk_event_free (event_copy);
+		}
+
+		return ret;
+	}
+
+	return GTK_WIDGET_CLASS (fm_icon_view_parent_class)->scroll_event (widget, scroll_event);
+}
+
 static void
 selection_changed_callback (NautilusIconContainer *container,
 			    FMIconView *icon_view)
@@ -2791,6 +2839,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	GTK_OBJECT_CLASS (klass)->destroy = fm_icon_view_destroy;
 	
 	GTK_WIDGET_CLASS (klass)->screen_changed = fm_icon_view_screen_changed;
+	GTK_WIDGET_CLASS (klass)->scroll_event = fm_icon_view_scroll_event;
 	
 	fm_directory_view_class->add_file = fm_icon_view_add_file;
 	fm_directory_view_class->flush_added_files = fm_icon_view_flush_added_files;
@@ -2952,7 +3001,6 @@ fm_icon_view_create (NautilusWindowSlotInfo *slot)
 			     "compact", FALSE,
 			     NULL);
 	g_object_ref (view);
-	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
 }
 
@@ -2966,7 +3014,6 @@ fm_compact_view_create (NautilusWindowSlotInfo *slot)
 			     "compact", TRUE,
 			     NULL);
 	g_object_ref (view);
-	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
 }
 
